// ******************* BASIC.FRAG *******************
// USING BLINN-PHONG

#version 330

struct Material {
	sampler2D diffuse;
	sampler2D specular;
	float shininess;
};

struct Light {
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;

	float constant;
	float linear;
	float quadratic;

	//Spotlight
	vec3 direction;
	float cutOff;
	float outerCutOff;
};

in vec2 TexCoords;
in vec3 Normal;
in vec3 FragPos;
in vec4 FragPosLightSpace;

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 brightcolor;

uniform vec3 viewPos;
uniform Material material;
uniform Light light;
uniform sampler2D shadowMap; //sampler2DShadow

vec3 projCoords;
float bias;
float diff;


float shadowCalculation(vec4 fragPosLightSpace)
{
	projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
	projCoords = projCoords * 0.5 + 0.5;
	float biast = max(0.05 * (1.0 - diff), 0.005); 
	float currentDepth = projCoords.z;
	
	/*
	float closestDepth = texture(shadowMap, projCoords.xy).r;
	//float shadow = (currentDepth - biast) > closestDepth  ? 1.0 : 0.3;
	if (currentDepth - biast > closestDepth)
		return 1.0;
	else
		return 0.0;
	*/
	
	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -2; x <= 2; ++x)
	{
		for(int y = -2; y <= 2; ++y)
		{
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r;
			shadow += currentDepth - biast > pcfDepth ? 0.8 : 0.0;
		}
	}
	shadow /= 25.0;
	return shadow;
}

void main()
{
	vec3 tcolor = texture(material.diffuse, TexCoords).rgb;
	
	// Attenuation
	float distance = length(light.position - FragPos);
	float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));

	// Ambient Light
	vec3 ambient = light.ambient;
	
	// Diffuse Light
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 halfwayDir = normalize(lightDir + viewDir);

	//Spotlight
	float theta = dot(lightDir, normalize(-light.direction));
	float epsilon = light.cutOff - light.outerCutOff;
	float intensity = clamp((theta - light.outerCutOff) / epsilon, 0.0, 1.0);
	
	vec3 diffuse;
	vec3 specular;
	vec3 result;

	vec3 ambientPart;
	vec3 diffusePart;
	vec3 specularPart;
	
	float shadow = shadowCalculation(FragPosLightSpace);

	diff = max(dot(lightDir, normalize(-light.direction)), 0.0);
	diffuse = light.diffuse * diff;
	
	// Specular Light
		// higher value means more concentrated highlight
		//Phong
		// vec3 viewDir = normalize(viewPos - FragPos);
		// vec3 reflectDir = reflect(-lightDir, norm);
	//Blinn-Phong
	float spec = pow(max(dot(norm, halfwayDir), 0.0), material.shininess);
	specular = light.specular * spec;

	ambientPart = ambient * tcolor * attenuation;
	diffusePart = diffuse * tcolor * attenuation;
	specularPart = specular * vec3(texture(material.specular, TexCoords)) * attenuation;
		
	
	result = ambientPart + (diffusePart + specularPart) * (intensity - shadow);

	color = vec4(result, 1.0f);

	// bright pass for bloom
	brightcolor = vec4(1.0f);
	brightcolor.r = max(color.r - 0.8f, 0.0f);
	brightcolor.g = max(color.g - 0.8f, 0.0f);
	brightcolor.b = max(color.b - 0.8f, 0.0f);
}